////////////////////////////////////////////////////////////
//
// class Fasta
//
// read fasta like input with sequence and structure
//
// (C) Sebastian Will, 2003
//
////////////////////////////////////////////////////////////

#include "fasta.hh"
#include <fstream>
#include <algo.h>

Fasta::Fasta(istream &in) {
  append(in);
}
Fasta::Fasta(const string &in) {
  append(in);
}
Fasta::Fasta(char *in) {
  append(in);
}

void Fasta::append(istream &in) {
  string n, s;
  vector<string> ps;
  
  while (!(ahead=="" && in.eof())) {
    read_seqstr(in,n,s,ps);
    names.push_back(n);
    sequences.push_back(s);
    structures.push_back(ps);
    ps.clear();
  }
}

void Fasta::append(const string &filename) {
  append(filename.c_str());
}
void Fasta::append(char *filename){
  ifstream in(filename);
  if (!in.is_open()) {
    cout <<"Cannot open "<<filename<<" for reading."<<endl;
    exit(-1);
  }
  append(in);
  in.close();
}

void filter(string &s, char c) {
  string::iterator new_end = remove(s.begin(),s.end(),c);
  s.resize(new_end-s.begin());
}

bool Fasta::read_seqstr(istream &in, string &n, string &s, vector<string> &ps) {
  string line=ahead;
  ahead="";
  string seqstr="";
  string strstr="";

  while ( !in.eof() && line == "") {
    std::getline(in,line);
    filter(line, ' ');
  }  
  
  if ( !in.eof() && line[0]=='>') {
    // cout << line<<endl;
    n = line;
    std::getline(in,line);
  } else {
    if (in.eof()) return true;
    cerr << "WARNING: sequence without name."<<endl;
  }

  do {
    filter(line, ' ');
    if (line.find("(")!=string::npos
	|| line.find(".")!=string::npos
	|| line.find(")")!=string::npos) break;
    
    seqstr.append(line);
  } while (std::getline(in,line));

  if (seqstr=="") return false; 
  s = seqstr;
  if (in.eof()) return true;
  
  do {
    filter(line, ' ');
    if (line[0]=='>'
	|| line.find("A")!=string::npos
	|| line.find("C")!=string::npos
	|| line.find("G")!=string::npos
	|| line.find("T")!=string::npos
	|| line.find("U")!=string::npos
	) break;
    
    strstr.append(line);
    if (strstr.length() >= seqstr.length()) {
      ps.push_back(strstr);
      strstr="";
    }
  } while (std::getline(in,line));
  
  if (!in.eof())
    ahead=line;
  
  return true;
}
