#ifndef FASTA_HH
#define FASTA_HH

////////////////////////////////////////////////////////////
//
// class Fasta
//
// read fasta like input with sequence and structure
//
// (C) Sebastian Will, 2003
//
////////////////////////////////////////////////////////////

#include <vector>
#include <string>
#include <iostream>

using namespace std;

class Fasta {
public:
  Fasta() {ahead="";}
  Fasta(istream &in);
  Fasta(const string &filename);
  Fasta(char *filename);
  
  // append name/sequence/structures entries
  void append(istream &in);
  void append(const string &filename);
  void append(char *filename);
  // void append(string name, string sequence, string structure);

  // read access
  const string &get_name(int i) const {return names[i];}
  const string &get_sequence(int i) const {return sequences[i];}
  const string &get_structure(int i, int j=0) const {return structures[i][j];}

  int num_sequences() const {return sequences.size();}
  int num_structures(int i=0) const {return structures[i].size();};

private:
  vector<string> names;
  vector<string> sequences;
  vector<vector<string> > structures;

  string ahead;

  void initialize(istream &in);

  bool read_seqstr(istream &in, string &n, string &s, vector<string> &ps);
};

#endif // FASTA_HH
