////////////////////////////////////////////////////////////
//
// A driver for LSSAligner
//
// call with --help option to get help on command line parameters,
// additionally: called without parameters:      reads from stdin
//               called with only one parameter: takes it as input file
//
// (C) Sebastian Will, 2003
//
////////////////////////////////////////////////////////////

#include <string>
#include <fstream>

#include "lssaligner.hh"
#include "fasta.hh"
extern "C" {
#include "options.h"
}

int main(int argc, char **argv) {

  // ------------------------------------------------------------
  // INPUT HANDLING
  //

  // ----------------------------------------
  // process options
 
  extern int optind;
  
  bool opt_help;
  bool opt_scoreonly;
  bool opt_tcoffee;
  bool opt_inputfile;
  bool opt_firstss;
  bool opt_secondss;
  
  char *inputfile;
  char *inputfile1;
  char *inputfile2;
  
  int tcoffee_mode=0;

  option_def the_options[] = {
    {"help",'h',&opt_help,O_NO_ARG,0,0,0,"This help"},
    {"scoreonly",'s',&opt_scoreonly,O_NO_ARG,0,0,0,"Print only score and do not compute alignment"},
    {"tcoffee",'t',&opt_tcoffee,O_ARG_INT,&tcoffee_mode,"0","mode","Output for T-Coffee"},
    {"seqstrfile",'f',&opt_inputfile,O_ARG_STRING,&inputfile,0,"infile","The input file"},
    {"seqstrfile1",'1',&opt_firstss,O_ARG_STRING,&inputfile1,0,"seq-str 1","First sequence-structure"},
    {"seqstrfile2",'2',&opt_secondss,O_ARG_STRING,&inputfile2,0,"seq-str 2","Second sequence-structure"},
    {0,0,0,0,0,0,0,0}
  };
 
  if (!process_options(argc,argv,the_options)) {
    if (!opt_help) {
      cout << "USAGE: ";
      print_usage(argv[0],the_options);
      cout <<endl;
      exit(-1);
    }
  }
 
  if (opt_help) {
    print_help(argv[0],the_options);
    exit(0);
  }

  // end process options
  // ----------------------------------------
  
  // ----------------------------------------
  // read sequence-structures

  sequence_t  s0;
  structure_t p0;
  
  sequence_t  s1;
  structure_t p1;
  
  
  if (argc==optind+4) {
    s0=string(argv[optind]);
    p0=string(argv[optind+1]);
    s1=string(argv[optind+2]);
    p1=string(argv[optind+3]);
  }
  else {
    if (argc==optind+1) {
      inputfile = argv[1];
      opt_inputfile=true;
    }
    if (argc>optind+1) {
      cout <<"USAGE: ";
      print_usage(argv[0],the_options);
      cout <<endl;
      exit(-1);
    }
    Fasta fasta;
    if (!opt_inputfile) {
      if (opt_firstss)
	fasta.append(inputfile1);
      else 
	fasta.append(cin);
      
      if (opt_secondss)
	fasta.append(inputfile2);
      else
	fasta.append(cin);
    }
    else {
      fasta.append(inputfile);
    }
    s0=fasta.get_sequence(0);
    p0=fasta.get_structure(0);
    s1=fasta.get_sequence(1);
    p1=fasta.get_structure(1);
  }
  
  if (s0.length() != p0.seqlength()
      ||
      s1.length() != p1.seqlength()) {
    cerr << "Lengthes of sequences and structure-strings do not fit."<<endl;
    exit(-1);
  }

  // end read sequence-structures
  // ----------------------------------------

  /*
  // ----------------------------------------
  // echo input sequences and structures
  cout <<s0<<endl;
  cout <<p0<<endl;
  cout <<s1<<endl;
  cout <<p1<<endl;
  */

  //
  // END INPUT HANDLING
  // ------------------------------------------------------------
  
  // ------------------------------------------------------------
  // ALIGNMENT COMPUTATION
  
  // ----------------------------------------
  // create and initialize aligner
  LSSAligner l(s0,p0,s1,p1);


  // ----------------------------------------
  // compute score
  score_t score = l.score();
  
  // ----------------------------------------
  // compute trace (optionally)
  if (!opt_scoreonly) l.traceback();
  
  
  // ----------------------------------------
  // output
  if (opt_scoreonly)
    cout << score << endl;
  else {
    if (opt_tcoffee) {
      l.print_tcoffee(score,tcoffee_mode);
    } else {
      l.print_alignment();
      cout <<endl<<"Best score: "<< score << endl;
    }
  }

  exit(0);
}
