#ifndef LSSAligner_HH
#define LSSAligner_HH

////////////////////////////////////////////////////////////
//
// class LSSAligner
//
// performs local sequence structure alignment
// computes similarity score and traceback
//
// (C) Sebastian Will, 2003
//
////////////////////////////////////////////////////////////


#define NDEBUG // undefining this, may turn on some expensive tests
const bool VERBOSE =
// true;
 false;

#include "arrays.hh"
#include "aux.hh"
#include <string.h>

typedef Sequence sequence_t;
typedef Structure structure_t;
typedef Alignment alignment_t;

typedef float score_t;
const score_t NEG_SCORE_INFTY = -1.0e20;

typedef array2<score_t> score_matrix_t;

#include "simscore.hh"

class LSSAligner {
public:
  LSSAligner(const sequence_t &s0_,const structure_t &p0_,
       const sequence_t &s1_, const structure_t &p1_):
    s0(s0_), s1(s1_), p0(p0_), p1(p1_),
    slen0(s0_.length()),
    slen1(s1_.length()),
    plen0(p0_.size()),
    plen1(p1_.size()),
    sim(simscore,s0,p0,s1,p1),
    AM(p0_.size(),p1_.size()),
    T(slen0+1,slen1+1)
  {
    for (int q=0; q<4; q++)
      M[q].resize(slen0+1,slen1+1);

  }
    
  score_t score();
  alignment_t traceback();

  void print_alignment();
  void print_tcoffee(score_t score, bool different_edge_weights) const;
  
private:

  score_t std_init_row(score_matrix_t &M, int a0l, int j1);
  score_t std_init_column(score_matrix_t &M, int j0, int a1l);
  
  score_t std_rec_cases(score_t maxscore, const score_matrix_t &s,
			int a0l, int a1l, int j0, int j1,
			int a0r, int a1r, Index *maxfrom);

  void special_rec_cases(const score_matrix_t s[], score_t maxscore[], 
			 int a0l,int a1l, int j0, int j1, 
			 int a0r, int a1r, Index *maxfrom);

  void compute_top_level(int i0, int i1, int r0, int r1);
  void compute_M_matrices(int a0l, int a1l, int a0r, int a1r);
  
  void trace_top_level(int i0, int i1, int j0, int j1);
  void trace_arc_match(int a0, int a1);
  void trace_M_matrices(int a0l, int a1l, int a0r, int a1r, int q, int j0, int j1);

  const sequence_t s0,s1;
  const structure_t p0,p1;

  int slen0,slen1,plen0,plen1;

  SimilarityScore sim;

  enum {ALLOW_BOTH_EXCLS, ALLOW_EXCL_IN_0, ALLOW_EXCL_IN_1, ALLOW_NO_EXCLS};
  
  score_matrix_t AM; // matrix for arc-matches
  score_matrix_t T; // matrix for top level
  score_matrix_t M[4]; // submatrices within arc-matches

  int max_l0, max_l1; // best start, computed by score()
  int max_i0, max_i1; // best end, computed by score()

  alignment_t alignment;
  
  bool is_edge_of_arcmatch(int i0, int i1) const;
  bool is_edge_of_arcbreak(int k, int i0, int i1) const;
};

#endif // LSSAligner_HH
