////////////////////////////////////////////////////////////
//
// classes SimilarityScore and SimilarityScoreConstants
//
// for defining a similarity score used by LSSAligner
// (actual parameters in simscoreconstants.hh)
//
// (C) Sebastian Will, 2003
//
////////////////////////////////////////////////////////////

#include "arrays.hh"

class SimilarityScoreConstants {
public:

  const score_t gap;
  const score_t match;
  const score_t mismatch;
  const score_t seq_factor;
  const score_t free_add;
  const score_t arcmatch_add;
  const score_t arcmatch_factor;
  const score_t arcbreak_add;
  const score_t arcbreak_factor;


SimilarityScoreConstants(score_t gap_, score_t match_, score_t mismatch_,
			 score_t seq_factor_, score_t free_add_,
			 score_t arcmatch_add_, score_t arcmatch_factor_, 
			 score_t arcbreak_add_, score_t arcbreak_factor_):
    gap(gap_),
    match(match_),
    mismatch(mismatch_),
    seq_factor(seq_factor_),
    free_add(free_add_),
    arcmatch_add(arcmatch_add_),
    arcmatch_factor(arcmatch_factor_),
    arcbreak_add(arcbreak_add_),
    arcbreak_factor(arcbreak_factor_) {}

};

class SimilarityScore {
  const SimilarityScoreConstants sim;
  const sequence_t s0,s1;
  const structure_t p0,p1;
  
  score_t seq_match(int i0, int i1) const {
    score_t s;
    if (s0[i0] == s1[i1]) s=sim.match; //match
    else s=sim.mismatch; // mismatch
    return s*sim.seq_factor;
  }
  
  score_t seq(int i0, int i1) const {
    if (i0==-1 || i1==-1) return sim.seq_factor * sim.gap; //gap
    else return seq_match(i0,i1);
  }
  
public:

  SimilarityScore(const SimilarityScoreConstants &sim_,
		  const sequence_t &s0_,const structure_t &p0_,
		  const sequence_t &s1_, const structure_t &p1_):
    sim(sim_),
    s0(s0_), s1(s1_), p0(p0_), p1(p1_) {}
    
  // ------------------------------------------------------------
  // similarity functions
  //  
  score_t base(int i0, int i1) const {
    if (i0==-1 || i1==-1) return sim.seq_factor * sim.gap;
    else return sim.free_add + seq_match(i0,i1);
  }

  score_t arc_match(int a0, int a1) const {
    return sim.arcmatch_add
      + sim.arcmatch_factor * (seq_match(p0[a0].first,p1[a1].first)
			       + seq_match(p0[a0].second,p1[a1].second));
  }

  // half similarity of an arc break in ss k, where the left end is i0, i1 
  score_t break_left(int k, int a, int i0, int i1) const {
    return sim.arcbreak_add + sim.arcbreak_factor*seq(i0,i1);
  }

  score_t break_right(int k, int a, int i0, int i1) const {
    return sim.arcbreak_add + sim.arcbreak_factor*seq(i0,i1);
  }
  //
  // end similarity functions
  // ------------------------------------------------------------

};

#include "simscoreconstants.hh"
