////////////////////////////////////////////////////////////
//
// actual parameters for similarity score used by LSSAligner
//
// this file is included from simscore.hh
// the constant >simscore< defines the parameters
//
//
// (C) Sebastian Will, 2003
//
////////////////////////////////////////////////////////////

/*
  Composition of similarity score from constants
  
  [i0,j0 are positions in the first sequence
   i1,j1 are positions in the second sequence]
  
  sim_base(i0,i1) =
    /
    | free_add, if (i0,i1) is a match or mismatch
    < 0,        otherwise
    \
    + seq_factor *
      /
      | match,    if (i0,i1) is a match
      < mismatch, if (i0,i1) is a mismatch
      | gap,      if (i0,i1) is a gap
      \

  sim_arcmatch((i0,j0),(i1,j1)) =
    arcmatch_add
    + arcmatch_factor * seq_factor * 
      (
        /
        | match,    if (i0,i1) is a match
        < mismatch, if (i0,i1) is a mismatch
        | gap,      if (i0,i1) is a gap
        \
	+	
	/
	| match,    if (j0,j1) is a match
        < mismatch, if (j0,j1) is a mismatch
        | gap,      if (j0,j1) is a gap
        \
      )

  break, where arc in first structure broken, left end
  sim_break_left((i0,j0),i1,j1) =
    break_add
    + break_factor * seq_factor * 
        /
        | match,    if (i0,i1) is a match
        < mismatch, if (i0,i1) is a mismatch
        | gap,      if (i0,i1) is a gap
        \
  
  Analogously:
   - right end
   - break, where arc in second structure broken
*/


static const SimilarityScoreConstants
simscore(
	 -0.7,   // gap
	 0.7,    // match
	 -0.4,   // mismatch
	 1.45,   // seq_factor
	 0.6,    // free_add
	 1.4,    // arcmatch_add
	 0.6,    // arcmatch_factor
	 -0.8,   // break_add
	 0.8     // break_factor
	 );
